calculate_reference_prior <- function(geod_pnts, 
                                      vn_phi = 1:10,
                                      bo_verbose = TRUE  # Print progress
)
  
{
  
  df_pnts <- as.data.frame(geod_pnts)
  
  ## "X"
  if (is.null(
    attr(df_pnts,
         "ncol.covariate")
  )
  )  # No covariates defined
  {
    mn_X <- matrix(1, 
                   ncol = 1, 
                   nrow = length(geod_pnts[["data"]]) 
    )
  } else # Covaraites are defined
  {
    mn_X <- as.matrix(
      cbind(1, df_pnts[,attr(df_pnts, "ncol.covariate")] )
    )
  }
  #mn_X <- mn_X[,1]
  
  
  vn_prior_phi <- NA
  
  mn_dist <-  as.matrix(
    dist(geod_pnts$coords,
         diag = TRUE,
         upper = TRUE
    )
  )
  
  #mn_dist[1:5,20:25]
  
  if(bo_verbose)
  {
    pb <- txtProgressBar(min = 1, 
                         max = length(vn_phi), 
                         style = 3)
  }
  
  i = 1
  for (n_phi in vn_phi)
  {
    if(bo_verbose)
    {
      setTxtProgressBar(pb, i)
    }  
    i = i+1
    
    ## "C^-1"
    mn_Cinv <- varcov.spatial(geod_pnts$coords,
                              cov.model = "exponential", 
                              #kappa     = 0.5, 
                              nugget    = 0,
                              scaled   = TRUE,  # Thus sigma sq is always 1
                              cov.pars  = c(0,n_phi),
                              inv       = TRUE
    )$inverse
    
    
    
    
    ## Partial derivative of C
    mn_der_cor <- exp(- mn_dist/ n_phi) * mn_dist * n_phi^(-2)
    
    #str(geod_pnts[[3]])
    
    
    # mn_W <- ma_der_cor %*% mn_Cinv %*% 
    #   (diag(x=1, nrow = nrow(mn_Cinv), ncol = ncol(mn_Cinv)) - 
    #      mn_X %*% 
    #      solve(t(mn_X) %*% mn_Cinv %*% mn_X)
    #     %*% t(mn_X)%*% mn_Cinv )
    #    
    # n_tra_W_sq <- sum(diag(mn_W %*% mn_W))
    # 
    # n_sq_tra_W <- sum(diag(mn_W))^2 
    # 
    # n_prior_phi <- sqrt(
    #   (676-1) * n_tra_W_sq - n_sq_tra_W 
    #   )
    
    ## Code partly from Tom Orton
    
    dAda <- mn_der_cor
    
    invA <-  mn_Cinv
    # dim(mn_Cinv)
    # dim(mn_X)
    invAX <-  mn_Cinv %*% mn_X
    XinvAX <- t(mn_X) %*% invAX
    
    tmp <- solve(XinvAX , t(invAX))
    TauMatXA <- invAX %*% tmp
    
    TA <- invA - TauMatXA
    
    Wa <- dAda %*% TA
    #WaTA <- Wa %*% TA
    WaWa <- Wa %*% Wa
    
    
    #trTA <- sum(diag(TA))
    #trTATA <- sum(diag(TATA))
    trWa <- sum(diag(Wa))
    #trWaTA <- sum(diag(WaTA))
    trWaWa <- sum(diag(WaWa))
    
    arg2 <- (nrow(mn_X) - ncol(mn_X)) * trWaWa - trWa ^ 2
    
    if (arg2 > 0)
    {
      n_prior_phi <- sqrt(arg2)  
    } else
    {
      n_prior_phi <- 0      
    }  
    
    vn_prior_phi <- c(vn_prior_phi, n_prior_phi)
    
  }
  
  if(bo_verbose)
  {
    close(pb)
  }  
  
  vn_prior_phi <- vn_prior_phi[-1]
  
  vn_prior_phi <- vn_prior_phi/sum(vn_prior_phi)
  
  return(vn_prior_phi)
  
  
}