compress_binomial_observations <- function(geod_cali, 
                                           vn_points_to_be_combined,
                                           bo_verbose = TRUE  # Print progress 
)
  # Note that this function currently cannot handle categorical covariates
{
  if( is.null(geod_cali$units.m) )
  {
    df_cali <- as.data.frame(geod_cali)
    
    n_covars <- ncol(df_cali) - 3
    
    df_cali[,4+n_covars] <- 1
    
    names(df_cali)[4+n_covars] <- "units.m"
    
    geod_cali <- as.geodata(obj        = df_cali,
                            coords.col = 1:2, 
                            data.col   = 3,
                            covar.col  = 4:(n_covars+3),
                            units.m.col = 4+n_covars)
    
    df_cali <- as.data.frame(geod_cali)
  } else
  {
    df_cali <- as.data.frame(geod_cali)
    n_covars <- ncol(df_cali) - 4
    
  }
  
  
  n_combining_iterations <- length(vn_points_to_be_combined)  
  
  
  
  #names(df_cali)[5] <- "data.1"
  
  ii <- 0
  
  for (n_points_to_be_combined in vn_points_to_be_combined)
  {
    
    ii <- ii + 1
    if(bo_verbose)
    {
      cat("Round ", ii, " of ", n_combining_iterations, "\n")
    }  
    
    
    #geod_cali
    di_01 <- dist(df_cali[, c("x", "y")],
                  diag = FALSE, 
                  upper = FALSE)
    
    #head(di_01)
    
    ma_distances <- as.matrix(di_01)
    
    #ma_distances[1:5, 1:5]
    
    vb_duplicate_part_of_matrix <- upper.tri(ma_distances, 
                                             diag = TRUE)
    
    ma_distances[vb_duplicate_part_of_matrix] <- NA
    
    remove(vb_duplicate_part_of_matrix)
    
    #vn_ordered_distances <- sort(di_01)
    
    
    
    # mn_loc_min <- which(x = (ma_distances <= vn_ordered_distances[n_points_to_be_combined] ),
    #                     arr.ind = TRUE )
    
    mn_loc <- which(x =  !is.na(ma_distances),
                    arr.ind = TRUE )
    
    
    mn_loc_and_dist <- cbind(mn_loc, distance = di_01)
    
    # head(mn_loc_and_dist)
    # 
    # vn_order_of_distances <- order(di_01)
    # 
    
    ## Order
    mn_loc_and_dist <- mn_loc_and_dist[order(mn_loc_and_dist[,"distance"]),]
    
    ## Prepare number of points to be combined
    ## Make sure not more points to be combined than there are points
    n_points_to_be_combined <- min(n_points_to_be_combined, nrow(df_cali))
    
    ## Make sure its an even number  
    n_points_to_be_combined <- floor(n_points_to_be_combined/2)*2
    
    ## Locations and distances
    
    #head(mn_loc_and_dist)
    
    # check
    # sqrt(
    #   (df_cali[18,"x"] - df_cali[17,"x"])^2 +
    #   (df_cali[18,"y"] - df_cali[17,"y"])^2 
    # )
    # 
    mn_loc_and_dist_final <- matrix(nrow = floor(n_points_to_be_combined/2), 
                                    ncol =3, 
                                    NA)
    
    if(bo_verbose)
    {
      pb <- txtProgressBar(min = 1, 
                           max = floor(n_points_to_be_combined/2), 
                           style = 3)
    }
    
    
    
    for (i in 1:floor(n_points_to_be_combined/2) )
    {
      if(bo_verbose)
      {
        setTxtProgressBar(pb, i)
      } 
      
      if(!is.matrix(mn_loc_and_dist)) # if mn_loc_and_dist appears to be a vector, not a matrix
        mn_loc_and_dist <- matrix(data = mn_loc_and_dist,
                                  nrow = 1)
      
      mn_loc_and_dist_final[i,] <- mn_loc_and_dist[1,]
      # remove duplicate locations
      vb_duplicated_locations <- mn_loc_and_dist[,1] %in% mn_loc_and_dist_final[i,1:2] |
        mn_loc_and_dist[,2] %in% mn_loc_and_dist_final[i,1:2] 
      #cat(sum(vb_duplicated_locations), "\n")
      # if(sum(vb_duplicated_locations)== 5)
      #   browser()
      mn_loc_and_dist <- mn_loc_and_dist[!vb_duplicated_locations,]
      #dim(mn_loc_and_dist)
      
    }
    
    
    
    
    vn_new_locations_x <- (df_cali[mn_loc_and_dist_final[,1],"x"] + df_cali[mn_loc_and_dist_final[,2],"x"])/2
    vn_new_locations_y <- (df_cali[mn_loc_and_dist_final[,1],"y"] + df_cali[mn_loc_and_dist_final[,2],"y"])/2 
    
    vn_new_locations_counts <- df_cali[mn_loc_and_dist_final[,1],"data"] + df_cali[mn_loc_and_dist_final[,2],"data"]
    vn_new_locations_draws  <- df_cali[mn_loc_and_dist_final[,1],"units.m"] + df_cali[mn_loc_and_dist_final[,2],"units.m"]
    
    
    df_new_locations_covariates <- data.frame(
      (df_cali[mn_loc_and_dist_final[,1],5:(n_covars+4)] + df_cali[mn_loc_and_dist_final[,2],5:(n_covars+4)])/2
    )     
    
    names(df_new_locations_covariates) <- names(df_cali)[5:(n_covars+4)]
    
    
    df_new_locations <- data.frame(x = vn_new_locations_x,
                                   y = vn_new_locations_y,
                                   data = vn_new_locations_counts,
                                   units.m = vn_new_locations_draws) 
    
    df_new_locations <- cbind(df_new_locations,
                              df_new_locations_covariates
    )
    
    
    #names(df_new_locations)[5] <- names(df_remaining_locations)[5]
    df_remaining_locations <- df_cali[-c(mn_loc_and_dist_final),]
    #browser()
    df_cali <- rbind(df_new_locations,
                     df_remaining_locations)
    
    
    if(bo_verbose)
    {
      close(pb)
    } 
    
    
  } # END for loop
  
  
  
  geod_cali <- as.geodata(obj        = df_cali,
                          coords.col = 1:2, 
                          data.col   = 3,
                          covar.col  = 5:ncol(df_cali),
                          units.m.col = 4)
  
  return(geod_cali)
}

